import 'dart:convert';
import 'dart:io';

import 'package:api_error_parser/api_error_parser.dart';
import 'package:flutter_test/flutter_test.dart';

import 'utils/entity/constants.dart';
import 'utils/entity/data_entity.dart';

ApiParser<String> parser = ApiParser(
  errorMessages: {
    ErrorCode.INSUFFICIENT_FUNDS: Message.EMPTY_BALANCE,
    ErrorCode.INVALID_PASSWORD_CONFIRMATION: Message.PASSWORD_DO_NOT_MATCH,
    ErrorCode.INVALID_PUNCTUATION: Message.PUNCTUATION_ERROR,
    ErrorCode.INVALID_PASSWORD: Message.INVALID_PASSWORD,
    ErrorCode.INVALID_LOGIN: Message.INVALID_LOGIN
  },
  fieldErrorMessages: {
    FIELD.EMAIL: {
      ErrorCode.INVALID_PASSWORD_CONFIRMATION: Message.PASSWORD_DO_NOT_MATCH,
    },
    FIELD.USER_PASSWORD: {
      ErrorCode.INVALID_PUNCTUATION: Message.PUNCTUATION_ERROR
    }
  },
  defaultErrorMessage: Message.DEFAULT,
  default403ErrorMessage: Message.DEFAULT_403_MESSAGE,
);

late ApiResponseEntity<DataEntity> response;

Future<ApiResponseEntity<DataEntity>> beforeTest() async {
  final file = File('test_resources/test.json');
  var jsonData = await file.readAsString();
  var decode = json.decode(jsonData);
  return ApiResponseEntity<DataEntity>.fromJson(
    decode,
    DataEntity.fromJson,
  );
}

void main() {
  setUp(() async {
    response = await beforeTest();
  });
  test("message from code", () {
    expect(
      parser.getMessageFromCode(ErrorCode.INVALID_PUNCTUATION),
      equals(Message.PUNCTUATION_ERROR),
    );
    expect(
      parser.getMessageFromCode(ErrorCode.UNKNOWN),
      equals(Message.DEFAULT),
    );
  });

  test("fits error message", () {
    final errors = response.errors;
    if (errors != null) {
      expect(
        parser.getFirstMessage(errors),
        equals(Message.EMPTY_BALANCE),
      );
      errors.insert(0, errors[2]);
      expect(
        parser.getFirstMessage(errors),
        equals(Message.DEFAULT),
      );
    }
  });

  test("error message", () {
    final errors = response.errors;
    if (errors != null) {
      expect(
          parser.getMessage(errors.last),
          equals(
            Message.INVALID_LOGIN,
          ));
      expect(
        parser.getMessage(errors[2]),
        equals(Message.DEFAULT),
      );
    }
  });

  test("list errors", () {
    final errors = response.errors;
    if (errors != null) {
      final parserErrors = parser.getErrors(errors);
      expect(errors[0].code, parserErrors[0].code);
      expect(errors[1].code, parserErrors[1].code);
      expect(errors[2].code, parserErrors[2].code);
      expect(errors[3].code, parserErrors[3].code);
      expect(errors[4].code, parserErrors[4].code);
      expect(errors[5].code, parserErrors[5].code);

      /* expect(parserErrors[0].message, Message.EMPTY_BALANCE);
      expect(parserErrors[1].message, Message.PUNCTUATION_ERROR);
      expect(parserErrors[2].message, Message.DEFAULT);
      expect(parserErrors[3].message, Message.PASSWORD_DO_NOT_MATCH);
      expect(parserErrors[4].message, Message.INVALID_PASSWORD);
      expect(parserErrors[5].message, Message.INVALID_LOGIN);*/

      expect(parserErrors[0].source == null, true);
      expect(parserErrors[1].source != null, true);
      expect(parserErrors[2].source != null, true);
      expect(parserErrors[3].source != null, true);
      expect(parserErrors[4].source != null, true);
      expect(parserErrors[5].source != null, true);
    }
  });

  test("parsing", () async {
    response = await beforeTest();
    final parserResponse = parser.getParserResponse(response);
    expect(response.data, parserResponse.data);

    final errors = response.errors;
    final parserErrors = parserResponse.errors;
    if (errors != null) {
      expect(errors[0].code, parserErrors[0].code);
      expect(errors[1].code, parserErrors[1].code);
      expect(errors[2].code, parserErrors[2].code);
      expect(errors[3].code, parserErrors[3].code);
      expect(errors[4].code, parserErrors[4].code);
      expect(errors[5].code, parserErrors[5].code);
    }
    /*expect(parserErrors[0].message, Message.EMPTY_BALANCE);
    expect(parserErrors[1].message, Message.PUNCTUATION_ERROR);
    expect(parserErrors[2].message, Message.DEFAULT);
    expect(parserErrors[3].message, Message.PASSWORD_DO_NOT_MATCH);
    expect(parserErrors[4].message, Message.INVALID_PASSWORD);
    expect(parserErrors[5].message, Message.INVALID_LOGIN);*/

    expect(parserErrors[0].source == null, true);
    expect(parserErrors[1].source != null, true);
    expect(parserErrors[2].source != null, true);
    expect(parserErrors[3].source != null, true);
    expect(parserErrors[4].source != null, true);
    expect(parserErrors[5].source != null, true);
  });

  test("api parser response", () async {
    response = await beforeTest();
    final errorResponse = parser.parse(response);
    expect(errorResponse is ApiParserErrorResponse, true);

    final errors = response.errors;
    final parserErrors = (errorResponse as ApiParserErrorResponse).errors;

    if (errors != null) {
      expect(errors[0].code, parserErrors[0].code);
      expect(errors[1].code, parserErrors[1].code);
      expect(errors[2].code, parserErrors[2].code);
      expect(errors[3].code, parserErrors[3].code);
      expect(errors[4].code, parserErrors[4].code);
      expect(errors[5].code, parserErrors[5].code);
    }

    /* expect(parserErrors[0].message, Message.EMPTY_BALANCE);
    expect(parserErrors[1].message, Message.PUNCTUATION_ERROR);
    expect(parserErrors[2].message, Message.DEFAULT);
    expect(parserErrors[3].message, Message.PASSWORD_DO_NOT_MATCH);
    expect(parserErrors[4].message, Message.INVALID_PASSWORD);
    expect(parserErrors[5].message, Message.INVALID_LOGIN);*/

    expect(parserErrors[0].source == null, true);
    expect(parserErrors[1].source != null, true);
    expect(parserErrors[2].source != null, true);
    expect(parserErrors[3].source != null, true);
    expect(parserErrors[4].source != null, true);
    expect(parserErrors[5].source != null, true);

    final emptyResponse = ApiResponseEntity(null, []);
    expect(parser.parse(emptyResponse) is ApiParserEmptyResponse, true);

    final successResponse = ApiResponseEntity<DataEntity>(
      response.data,
      [],
    );
    expect(
      parser.parse(successResponse) is ApiParserSuccessResponse,
      true,
    );
    expect(successResponse.data != null, true);
    expect(successResponse.data, response.data);
  });
}
